# Blog Guidelines

## Introduction

Welcome to the Security TAG (STAG) Blog! Our blog is a platform for sharing insights, experiences, and knowledge about Cloud Native software and the community. We encourage contributions from community members to help educate, inspire, and engage others. To ensure quality and consistency, we have established the following guidelines for content creation, submission review, and the overall value to the community.

## Proposal

Everyone is welcome to propose a new blog post using the [blog submission template](https://github.com/cncf/tag-security/blob/main/.github/ISSUE_TEMPLATE/blog-submission.md). Blog article proposals are meant to capture the intent of authoring one of many topics covered below.

## Content Creation

### Topics

- **Cloud Native Projects**: Share updates, features, and success stories.
- **Technical Tutorials**: Provide how-to guides, coding tips, and best practices.
- **Community Stories**: Highlight contributors, events, and community achievements.
- **Working Groups**: Proposed initiatives, Updates, and calls to action.
- **Opinion Pieces**: Discuss trends, challenges, and the future of open source.

### Writing Style

- **Clarity**: Write in clear, concise language. Avoid jargon unless it's well-explained.
- **Engagement**: Make your post interesting and engaging. Use anecdotes and examples.
- **Accuracy**: Ensure all information is accurate and up-to-date.
- **Inclusivity**: Use inclusive language and be respectful to all readers.

### Formatting

- **File Name**: Blog Files should be dash-case `.md` markdown filetype.
- **Headings**: Use appropriate headings (H1, H2, H3, etc.) to structure your post.
- **Paragraphs**: Keep paragraphs short and focused.
- **Lists**: Use bullet points or numbered lists to highlight key points.
- **Tags**: Include relevant tags in the blog for filtering capabilities.
- **Code**: Use code blocks for any code snippets.
- **Images**: Include relevant images and graphics to enhance the content.

### Front Matter

The start of each blog document should contain the `frontmatter` information.

Required `frontmatter` fields:

- **title**: Title-case title for the blog post
- **author**: Name of blog post author

Note: **date** can be included but will be appended to the `frontmatter` when publication date is decided.

#### Example

```yaml
---
title:  "New Informative Blog Post"
date:   2024-06-26 09:00:00 -0700
author: R. Raccoon
---
```

### Length

- Aim for 500-1200 words. Ensure the content is comprehensive yet concise.

## Submission Review Process

### Submission

1. **Draft**: Write your blog post in a markdown file under `tag-security/website/content/blog`.
2. **Pull Request**: Submit your draft in a pull request.

### Review

1. **Initial Review**: The collective team will perform an initial review.
2. **Feedback**: You may receive feedback and requests for revisions.
3. **Revisions**: Address the feedback and resubmit your revised draft.
4. **Final Review**: The team conducts a final review to ensure quality.

### Reviewer Criteria

1. **Content**: Submission abides by all `Content Creation` guidelines established above.
2. **Sponsor**: Submission is sponsored by a STAG Co-Chair, Tech Lead, or Project Lead.
3. **Existing Content**: Submission does not duplicate existing material.

### Approval

- **Acceptance**: Once approved, your post will be scheduled for publication.
- **Notification**: You will be notified of the publication date - a repo maintainer may update this field prior to merge.
- **Merge**: A repository maintainer will perform the merge once accepted.

## Value to the Community

### Education

- **Knowledge Sharing**: Help others learn new skills and techniques.
- **Resources**: Provide valuable resources and references.

### Inspiration

- **Success Stories**: Inspire others with success stories and case studies.
- **Innovation**: Encourage innovative thinking and solutions.

### Engagement

- **Discussion**: Foster discussions and interactions within the community.
- **Support**: Offer support and encouragement to fellow community members.

### Growth

- **Visibility**: Gain visibility for your contributions and projects.
- **Network**: Call for contribution to get others involved in initiatives.

## Conclusion

Thank you for your interest in contributing to the STAG Blog. By following these guidelines, you help maintain the quality and integrity of our content, benefiting the entire community. We look forward to your valuable contributions!
